@@ -16,7 +16,7 @@
 #
 #  Basic io.
 #
-#  $Id: rpy_io.py,v 1.5 2004/02/27 17:05:40 warnes Exp $
+#  $Id: rpy_io.py,v 1.6 2005/07/14 06:37:57 warnes Exp $
 #
 #
 
@@ -25,7 +25,9 @@ import os
 
 # Input and output to sys.std{in,out}
 def rpy_output(s):
+    s = s.replace('_\b','')
     sys.stdout.write(s)
+    sys.stdout.flush()
 
 def rpy_input(prompt, n):
     rpy_output(prompt)
