@@ -16,7 +16,7 @@
  *
  *  Evaluation of R expressions.
  *
- *  $Id: R_eval.c,v 1.11 2004/10/22 13:06:13 warnes Exp $
+ *  $Id: R_eval.c,v 1.12 2005/07/27 03:05:30 warnes Exp $
  *
  */
 
@@ -87,7 +87,7 @@ SEXP do_eval_expr(SEXP e) {
 SEXP do_eval_fun(char *name) {
   SEXP exp, fun, res;
 
-  fun = get_from_name(name);
+  fun = get_fun_from_name(name);
   if (!fun)
     return NULL;
 
@@ -101,11 +101,11 @@ SEXP do_eval_fun(char *name) {
 }
 
 /*
- * Get an R object from his name. When not found, an exception is
+ * Get an R **function** object by its name. When not found, an exception is
  * raised. The checking of the length of the identifier is needed to
- * avoid that R raises an error. If this happens, Python dumps core.
+ * avoid R raising an error causing Python to  dump core.
  */
-SEXP get_from_name(char *ident) {
+SEXP get_fun_from_name(char *ident) {
   SEXP obj;
 
   /* For R not to throw an error, we must check the identifier is
@@ -118,17 +118,31 @@ SEXP get_from_name(char *ident) {
     PyErr_SetString(RPyExc_Exception, "symbol print-name too long");
     return NULL;
   }
+  
+#if R_VERSION < 0x20000
+  obj = Rf_findVar(Rf_install(ident), R_GlobalEnv);
+#else
+  /*
+   * For R-2.0.0 and later, it is necessary to use findFun to get
+   * functions.  Unfortunately, calling findFun on an undefined name
+   * causes a segfault!
+   *
+   * Solution:
+   *
+   * 1) Call findVar on the name
+   *
+   * 2) If something has the name, call findFun
+   *
+   * 3) Raise an error if either step 1 or 2 fails.
+   */
+  obj = Rf_findVar(Rf_install(ident), R_GlobalEnv);
 
-#if R_VERSION < 0x20000   // pre-R-2.0.0
-  obj = Rf_findVar(Rf_install(ident), R_GlobalEnv);
-#else  
-  obj = Rf_findVar(Rf_install(ident), R_GlobalEnv);
-  if (obj != R_UnboundValue) 
+  if (obj != R_UnboundValue)
       obj = Rf_findFun(Rf_install(ident), R_GlobalEnv);
 #endif
-
+  
   if (obj == R_UnboundValue) {
-    PyErr_Format(RPyExc_Exception, "Object \"%s\" not found", ident);
+    PyErr_Format(RPyExc_Exception, "R Function \"%s\" not found", ident);
     return NULL;
   }
   return obj;
