@@ -16,7 +16,7 @@
  *
  *  Implementation of the module '_rpy' and the 'Robj' type.
  *
- *  $Id: rpymodule.c,v 1.27 2005/07/20 05:38:03 warnes Exp $
+ *  $Id: rpymodule.c,v 1.31 2005/07/28 14:23:33 warnes Exp $
  *
  */
 
@@ -28,10 +28,18 @@
 #include <stdlib.h>
 #include <string.h>
 
+/* Flag indicating whether Numeric is available in this session
+ *
+ * This is necessary Numeric may not available at run time, even if
+ * it was available at compile time.
+*/
+static int use_numeric=0;
+
+
 /* Local function definitions */
-DL_EXPORT(void) INIT_RPY(void);   /* Module initializer */
-static PyObject *r_init(void);    /* Class initializer */
-static PyObject *r_cleanup(void); /* Clean up R & release resources */
+DL_EXPORT(void) INIT_RPY(void);                     /* Module initializer */
+static PyObject *r_init(PyObject *self, PyObject *args); /* Class initializer */
+static PyObject *r_cleanup(void);                    /* Clean up R & release resources */
 
 
 /* Global objects */
@@ -370,7 +378,7 @@ to_Robj(PyObject *obj)
   else if (PyInt_Check(obj))
     {
       PROTECT(robj = NEW_INTEGER(1));
-      INTEGER_DATA(robj)[0] = PyInt_AsLong(obj);
+      INTEGER_DATA(robj)[0] = (int) PyInt_AsLong(obj);
     }
   else if (PyFloat_Check(obj)) 
     {
@@ -388,13 +396,14 @@ to_Robj(PyObject *obj)
     {
       PROTECT(robj = NEW_STRING(1));
       SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(obj)));
-#ifdef WITH_NUMERIC
     }
+#ifdef WITH_NUMERIC
+  //  else if (use_numeric && PyArray_Check(obj)) 
   else if (PyArray_Check(obj)) 
     {
       PROTECT(robj = to_Rarray(obj));
-#endif
     }
+#endif
   else if ((PySequence_Check(obj)) &&
            (PySequence_Size(obj) >= 0)) 
     {
@@ -434,6 +443,7 @@ to_PyDict(PyObject *obj, SEXP names)
   dict = PyDict_New();
   for (i=0; i<len; i++) {
     if (!(it = PyList_GetItem(obj, i)))
+      
       return NULL;
     name = CHAR(STRING_ELT(names, i));
     if ((PyDict_SetItemString(dict, name, it)) < 0) {
@@ -666,13 +676,16 @@ to_Pyobj_vector(SEXP robj, PyObject **obj, int mode)
   dim = GET_DIM(robj);
   if (dim != R_NilValue) {
 #ifdef WITH_NUMERIC
-    array = to_PyNumericArray(tmp, dim);
-    if (array) {                /* If the conversion to Numeric succeed.. */
-      *obj = array;             /* we are done */
-      Py_DECREF(tmp);
-      return 1;
-    }
-    PyErr_Clear();
+    if(use_numeric)
+      {
+        array = to_PyNumericArray(tmp, dim);
+        if (array) {                /* If the conversion to Numeric succeed.. */
+          *obj = array;             /* we are done */
+          Py_DECREF(tmp);
+          return 1;
+        }
+        PyErr_Clear();
+      }
 #endif
     len = GET_LENGTH(dim);
     *obj = to_PyArray(tmp, INTEGER(dim), len);
@@ -961,6 +974,9 @@ Robj_call(PyObject *self, PyObject *args, PyObject *kwds)
 
   obj = to_Pyobj_with_mode(res, conv);
   UNPROTECT(2);
+
+  PrintWarnings(); /* show any warning messages */
+  
   return obj;
 }
 
@@ -1356,7 +1372,7 @@ PyTypeObject Robj_Type = {
 /* Obtain an R object via its name. 'autoconvert' is the keyword to
    set the autoconversion flag. */
 static PyObject *
-get(PyObject *self, PyObject *args, PyObject *kwds)
+get_fun(PyObject *self, PyObject *args, PyObject *kwds)
 {
   char *obj_str;
   int conversion=TOP_MODE;
@@ -1367,7 +1383,7 @@ get(PyObject *self, PyObject *args, PyObject *kwds)
                                    &obj_str, &conversion))
     return NULL;
 
-  robj = get_from_name(obj_str);
+  robj = get_fun_from_name(obj_str);
   if (!robj)
     return NULL;
 
@@ -1478,9 +1494,8 @@ void r_finalize(void)
       snprintf((char *)buf, 1024, "rm -rf %s", tmpdir); 
       R_system((char *)buf);                            
     }
-    if(R_CollectWarnings)
-      PrintWarnings();	/* from device close and .Last */
-                                                        
+
+    PrintWarnings();	/* from device close and .Last */
     R_gc();  /* Remove any remaining R objects from memory */
 }
 
@@ -1493,10 +1508,39 @@ r_cleanup(void)
   return Py_None;
 }
 
+#ifdef WITH_NUMERIC
+static void
+init_numeric(void)
+{
+  PyObject *multiarray, *dict;
+  
+  if(use_numeric)
+    {
+      import_array();
+      multiarray = PyImport_ImportModule("multiarray");
+      if (multiarray) {
+        dict = PyModule_GetDict(multiarray);
+        if (dict)
+          Py_transpose = PyDict_GetItemString(dict, "transpose");
+      }
+    }
+}
+#endif
+
 static PyObject *
-r_init(void)
+r_init(PyObject *self, PyObject *args)
 {
   static int first=1;
+  int i;
+  
+  if (!PyArg_ParseTuple(args, "i:r_init", &i))
+    return NULL;
+  use_numeric = i;
+
+#ifdef WITH_NUMERIC
+  init_numeric();
+#endif
+  
   if(first==1)
     {
       first=0;
@@ -1512,7 +1556,7 @@ r_init(void)
 
 /* List of functions defined in the module */
 static PyMethodDef rpy_methods[] = {
-  {"get",           (PyCFunction)get,           METH_VARARGS | METH_KEYWORDS},
+  {"get_fun",       (PyCFunction)get_fun,           METH_VARARGS | METH_KEYWORDS},
   {"set_mode",      (PyCFunction)set_mode,      METH_VARARGS},
   {"get_mode",      (PyCFunction)get_mode,      METH_VARARGS},
   {"set_output",    (PyCFunction)set_output,    METH_VARARGS},
@@ -1523,7 +1567,7 @@ static PyMethodDef rpy_methods[] = {
   {"get_showfiles", (PyCFunction)get_showfiles, METH_VARARGS},
   {"r_events",      (PyCFunction)r_events,      METH_VARARGS | METH_KEYWORDS},
   {"r_cleanup",     (PyCFunction)r_cleanup,     METH_NOARGS},
-  {"r_init",        (PyCFunction)r_init,        METH_NOARGS},
+  {"r_init",        (PyCFunction)r_init,        METH_VARARGS},
   {NULL, NULL}          /* sentinel */
 };
 
@@ -1642,7 +1686,7 @@ init_embedded_win32( void ) {
   Rp->R_Slave = Rp->R_Verbose = 0;
   Rp->R_Interactive = TRUE;
   Rp->RestoreAction = SA_RESTORE; /* no restore */
-  Rp->SaveAction    = SA_NOSAVE;    /* no save */
+  Rp->SaveAction    = SA_NOSAVE;  /* no save */
   
 #if R_VERSION < 0x20000   // pre-R-2.0.0
 
@@ -1675,9 +1719,6 @@ INIT_RPY(void)
 #endif
   SEXP interact;
 
-#ifdef WITH_NUMERIC
-  PyObject *multiarray, *dict;
-#endif
 
   Robj_Type.ob_type = &PyType_Type;
 #if defined( _WIN32 ) && ! defined( PRE_2_2 )
@@ -1686,15 +1727,6 @@ INIT_RPY(void)
 #endif
 
   m = Py_InitModule(RPY_SHNAME, rpy_methods);
-#ifdef WITH_NUMERIC
-  import_array();
-  multiarray = PyImport_ImportModule("multiarray");
-  if (multiarray) {
-    dict = PyModule_GetDict(multiarray);
-    if (dict)
-      Py_transpose = PyDict_GetItemString(dict, "transpose");
-  }
-#endif
   d = PyModule_GetDict(m);
 
   /* Save this interpreter */
@@ -1750,12 +1782,12 @@ INIT_RPY(void)
   PyDict_SetItemString(d, "__proc_table__", proc_table);
 
   // The globals R objects for the sequence protocol
-  get_item = get_from_name("[");
-  set_item = get_from_name("[<-");
-  length = get_from_name("length");
+  get_item = get_fun_from_name("[");
+  set_item = get_fun_from_name("[<-");
+  length = get_fun_from_name("length");
 
   // Function to transpose arrays
-  aperm = get_from_name("aperm");
+  aperm = get_fun_from_name("aperm");
 
   // Initialize the list of protected objects
   R_References = R_NilValue;
