@@ -21,6 +21,9 @@ class foo:
 
 class TypeConversionToRTestCase(unittest.TestCase):
 
+    def setup(self):
+        set_default_mode(NO_DEFAULT)
+
     def testRobjToR(self):
         def f():
             r1 = r.c(4)
